@@ -171,6 +171,10 @@ class StringShape(Shape):
     def enum(self):
         return self._shape_data.get('enum', [])
 
+    @CachedProperty
+    def deprecated_enum_values(self):
+        return self._shape_data.get('x-deprecated-enum-values', [])
+
     @CachedProperty
     def supported_options(self):
         return self._shape_data.get('x-supported-options', [])
@@ -358,6 +362,14 @@ class ServiceModel(object):
                                           operation_name,
                                           http_method,
                                           request_uri)
+                if operation_data.get("x-alt-operation-id", None) == operation_name:
+                    operation_data = operation_data.copy()
+                    operation_data["x-deprecated"] = True
+                    return OperationModel(operation_data,
+                                          self,
+                                          operation_name,
+                                          http_method,
+                                          request_uri)
         raise OperationNotFoundError(operation_name)
 
     @CachedProperty
@@ -365,8 +377,10 @@ class ServiceModel(object):
         operation_names = []
         for path in self._service_data["paths"]:
             for operation in self._service_data["paths"][path]:
-                operation_names.append(
-                    self._service_data["paths"][path][operation]["operationId"])
+                operation_data = self._service_data["paths"][path][operation]
+                operation_names.append(operation_data["operationId"])
+                if "x-alt-operation-id" in operation_data:
+                    operation_names.append(operation_data["x-alt-operation-id"])
         return operation_names
 
     def resolve_shape_ref(self, name, shape_ref):
@@ -492,6 +506,8 @@ class DenormalizedStructureBuilder(object):
             shape['documentation'] = model['documentation']
         if 'enum' in model:
             shape['enum'] = model['enum']
+        if 'x-deprecated-enum-values' in model:
+            shape['x-deprecated-enum-values'] = model['x-deprecated-enum-values']
         if 'x-supported-options' in model:
             shape['x-supported-options'] = model['x-supported-options']
         if 'x-no-paramfile' in model:
